@@ -1,33 +1,50 @@
-use super::subprocess::SubprocessCommand;
-use super::types::Fail;
+use super::{subprocess::SubprocessCommand, types::Fail};
 use aho_corasick::{AhoCorasick, AhoCorasickBuilder};
+use clap::Parser;
 use regex::{Regex, RegexBuilder};
 use shlex::split;
-use std::{collections::HashMap, env, path::PathBuf};
-use structopt::StructOpt;
+use std::{
+  collections::HashMap,
+  env::{args_os, current_dir, var_os},
+  path::PathBuf,
+};
+
 use which::which;
 
-#[derive(Debug, StructOpt)]
-#[structopt(name = "sad", author, about)]
+#[derive(Debug)]
+pub enum Mode {
+  Initial,
+  Preview(PathBuf),
+  Patch(PathBuf),
+}
+
+impl Mode {
+  pub const ARGV: &'static str = env!("SAD_ARGV_UUID");
+  pub const PREVIEW: &'static str = env!("SAD_PREVIEW_UUID");
+  pub const PATCH: &'static str = env!("SAD_PATCH_UUID");
+}
+
+#[derive(Debug, Parser)]
+#[clap(about, version)]
 pub struct Arguments {
   /// Search pattern
-  #[structopt()]
+  #[clap()]
   pub pattern: String,
 
   /// Replacement pattern, empty = delete
-  #[structopt()]
+  #[clap()]
   pub replace: Option<String>,
 
   /// Use \0 as stdin delimiter
-  #[structopt(short = "0", long = "read0")]
-  pub nul_delim: bool,
+  #[clap(short = '0', long)]
+  pub read0: bool,
 
   /// No preview, write changes to file
-  #[structopt(short = "k", long)]
+  #[clap(short = 'k', long)]
   pub commit: bool,
 
   /// String literal mode
-  #[structopt(short, long)]
+  #[clap(short, long)]
   pub exact: bool,
 
   /// Regex flags: use `--help` instead of `-h` to see details
@@ -43,49 +60,50 @@ pub struct Arguments {
   /// u :: swap the meaning of '*' and '*?' (lazy & greedy matching)
   ///
   /// x :: ignore whitespaces and '#' comments
-  #[structopt(short, long)]
+  #[clap(short, long)]
   pub flags: Option<String>,
 
   /// Colourizing program, disable = never, default = $GIT_PAGER
   ///
   /// Uses bash shell syntax for splitting
-  #[structopt(short, long)]
+  #[clap(short, long)]
   pub pager: Option<String>,
 
   /// Additional Fzf options, disable = never
   ///
   /// Uses bash shell syntax for splitting
-  #[structopt(long)]
+  #[clap(long)]
   pub fzf: Option<String>,
 
   /// Same as in GNU diff --unified={size}, affects aggregate size
   ///
   /// ie. a higher {size} will leader to more changes grouped together
-  #[structopt(short, long)]
+  #[clap(short, long)]
   pub unified: Option<usize>,
-
-  /// *Internal use only*
-  #[structopt(long)]
-  pub internal_preview: Option<PathBuf>,
-
-  /// *Internal use only*
-  #[structopt(long)]
-  pub internal_patch: Option<PathBuf>,
 }
 
-pub fn parse_args() -> Result<Arguments, Fail> {
-  let args = env::args().collect::<Vec<_>>();
-  match (args.get(1), args.get(2)) {
-    (Some(lhs), Some(rhs)) if lhs == "-c" => {
-      if rhs.contains('\x04') {
-        Ok(Arguments::from_iter(rhs.split('\x04')))
-      } else {
-        Err(Fail::ArgumentError(
-          "`-c` is a reserved flag, use --k, or --commit".to_owned(),
-        ))
+pub fn parse_args() -> Result<(Mode, Arguments), Fail> {
+  let args = args_os().collect::<Vec<_>>();
+  match (
+    args.get(1).and_then(|a| a.to_str()),
+    args.get(2).and_then(|a| {
+      let exec = a
+        .to_str()
+        .unwrap_or_default()
+        .split('\x04')
+        .collect::<Vec<_>>();
+      match exec[..] {
+        [Mode::PREVIEW, path] => Some(Mode::Preview(PathBuf::from(path))),
+        [Mode::PATCH, path] => Some(Mode::Patch(PathBuf::from(path))),
+        _ => None,
       }
+    }),
+    var_os(Mode::ARGV).and_then(|a| a.into_string().ok()),
+  ) {
+    (Some("-c"), Some(mode), Some(arg_list)) => {
+      Ok((mode, Arguments::parse_from(arg_list.split('\x04'))))
     }
-    _ => Ok(Arguments::from_args()),
+    _ => Ok((Mode::Initial, Arguments::parse_from(args))),
   }
 }
 
@@ -99,7 +117,7 @@ pub enum Engine {
 pub enum Action {
   Preview,
   Commit,
-  Fzf(PathBuf, Vec<String>),
+  FzfPreview(PathBuf, Vec<String>),
 }
 
 #[derive(Clone, Debug)]
@@ -139,8 +157,7 @@ fn p_aho_corasick(pattern: &str, flags: Vec<String>) -> Result<AhoCorasick, Fail
       "I" => ac.ascii_case_insensitive(false),
       _ => {
         return Err(Fail::ArgumentError(format!(
-          "Invaild regex flag, see `--help` :: {}",
-          flag
+          "Invaild regex flag, see `--help` :: {flag}"
         )))
       }
     };
@@ -164,8 +181,7 @@ fn p_regex(pattern: &str, flags: Vec<String>) -> Result<Regex, Fail> {
       "X" => re.ignore_whitespace(false),
       _ => {
         return Err(Fail::ArgumentError(format!(
-          "Invaild regex flag, see `--help` :: {}",
-          flag
+          "Invaild regex flag, see `--help` :: {flag}"
         )))
       }
     };
@@ -175,9 +191,9 @@ fn p_regex(pattern: &str, flags: Vec<String>) -> Result<Regex, Fail> {
 
 fn p_fzf(fzf: Option<String>) -> Option<(PathBuf, Vec<String>)> {
   match (which("fzf"), atty::is(atty::Stream::Stdout)) {
-    (Ok(p), true) => match fzf {
-      Some(val) if val == "never" => None,
-      Some(val) => Some((p, split(&val).unwrap_or_default())),
+    (Ok(p), true) => match fzf.as_deref() {
+      Some("never") => None,
+      Some(val) => Some((p, split(val).unwrap_or_default())),
       None => Some((p, Vec::new())),
     },
     _ => None,
@@ -186,21 +202,23 @@ fn p_fzf(fzf: Option<String>) -> Option<(PathBuf, Vec<String>)> {
 
 fn p_pager(pager: &Option<String>) -> Option<SubprocessCommand> {
   let norm = || which("delta").or_else(|_| which("diff-so-fancy")).ok();
-  let (prog, arguments) = match pager {
-    Some(val) => match val as &str {
-      "never" => (None, Vec::new()),
-      _ => {
-        let mut sh = split(val)
-          .unwrap_or_else(|| vec![val.to_owned()])
-          .into_iter();
-        (
-          sh.next().and_then(|p| which(p).ok()).or_else(norm),
-          sh.collect(),
-        )
-      }
-    },
+
+  let (prog, arguments) = match pager.as_deref() {
+    Some("never") => (None, Vec::new()),
+    Some(val) => {
+      let mut sh = split(val)
+        .unwrap_or_else(|| vec![val.to_owned()])
+        .into_iter();
+      (
+        sh.next().and_then(|p| which(p).ok()).or_else(norm),
+        sh.collect(),
+      )
+    }
     None => {
-      let val = env::var("GIT_PAGER").unwrap_or_default();
+      let val = var_os("GIT_PAGER")
+        .and_then(|v| v.into_string().ok())
+        .unwrap_or_default();
+
       let less_less = val.split('|').next().unwrap_or(&val).trim();
       let mut sh = split(less_less)
         .unwrap_or_else(|| vec![less_less.to_owned()])
@@ -219,7 +237,7 @@ fn p_pager(pager: &Option<String>) -> Option<SubprocessCommand> {
   })
 }
 
-pub fn parse_opts(args: Arguments) -> Result<Options, Fail> {
+pub fn parse_opts(mode: Mode, args: Arguments) -> Result<Options, Fail> {
   let mut flagset = p_auto_flags(args.exact, &args.pattern);
   flagset.extend(
     args
@@ -239,14 +257,11 @@ pub fn parse_opts(args: Arguments) -> Result<Options, Fail> {
     }
   };
 
-  let action = if args.commit || args.internal_patch != None {
-    Action::Commit
-  } else {
-    match (args.internal_preview, p_fzf(args.fzf)) {
-      (Some(_), _) => Action::Preview,
-      (_, None) => Action::Preview,
-      (_, Some((bin, args))) => Action::Fzf(bin, args),
-    }
+  let action = match (args.commit, mode, p_fzf(args.fzf)) {
+    (true, _, _) => Action::Commit,
+    (_, Mode::Patch(_), _) => Action::Commit,
+    (_, Mode::Initial, Some((bin, args))) => Action::FzfPreview(bin, args),
+    _ => Action::Preview,
   };
 
   let printer = match p_pager(&args.pager) {
@@ -255,7 +270,7 @@ pub fn parse_opts(args: Arguments) -> Result<Options, Fail> {
   };
 
   Ok(Options {
-    cwd: env::current_dir().ok(),
+    cwd: current_dir().ok(),
     action,
     engine,
     printer,
